#!@PERL@

# --8<--8<--8<--8<--
#
# Copyright (C) 2010 Smithsonian Astrophysical Observatory
#
# This file is part of make-world
#
# make-world is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# make-world is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#       Free Software Foundation, Inc.
#       51 Franklin Street, Fifth Floor
#       Boston, MA  02110-1301, USA
#
# -->8-->8-->8-->8--

#================================================================#
# redirect STDERR, STDOUT

use strict;
use warnings;

package MakeWorld::IORedir;

my ( $ofh, $STDOUT, $STDERR );


# backup real io streams
BEGIN {
    open $STDOUT, '>&', STDOUT;
    open $STDERR, '>&', STDERR;
}

# make a backup of the original filehandles
INIT { $ofh = MakeWorld::IORedir->new() };

sub init {

    my $class = shift;

    # first destroy original $ofh.  if this isn't done,
    # it'll get destroyed upon assignment, *after* creating the new
    # one.  this will reset the current fd's to $ofh's saved one
    # *after* the new one has set the, thus confusing things.
    undef $ofh;

    $ofh = $class->new( @_ );

    # now STDOUT and STDERR are set to what is requested, but we also
    # need $ofh to believe those are the values to be used when
    #   MakeWorld::IORedir->new( reset => 1 ) is called
    $ofh->_save;

    return;
}

sub new {

    my ( $class, %attr ) = @_;

    my $self = bless {}, $class;

    $attr{append} //= 1;

    # save current file handles
    $self->_save();


    if ( defined $attr{file} )
    {
	open ( STDOUT, $attr{append} ? '>>' : '>', $attr{file} )
	  or die( "can't redirect STDOUT to $attr{file}\n" );

	open ( STDERR, '>&', STDOUT)
	  or do{
	      open STDOUT, ">&", delete $self->{stdout}
		or die( "error restoring STDOUT\n" );
	      die( "can't redirect STDERR to $attr{file}\n" );
	  };
    }
    elsif ( defined $attr{reset} )
    {
	open STDOUT, ">&", $ofh->{stdout}
	  or die( "error resetting STDOUT\n" );

	open STDERR, ">&", $ofh->{stderr}
	      or die( "error resetting STDERR\n" );
    }

    elsif ( defined $attr{original} )
    {
	open STDOUT, ">&", $STDOUT
	  or die( "error resetting STDOUT\n" );

	open STDERR, ">&", $STDERR
	      or die( "error resetting STDERR\n" );
    }

    return $self;
}

sub _save
{
    my ( $self ) = @_;

    open $self->{stdout}, ">&STDOUT"
      or die( "Can't dup STDOUT: $!\n" );

    open $self->{stderr}, ">&STDERR"
      or die( "Can't dup STDERR: $!\n" );
}


sub DESTROY {
    my $self = shift;

    if ( exists $self->{stdout} )
    {
	open STDOUT, ">&", delete $self->{stdout}
	  or die( "error restoring STDOUT\n" );
    }

    if ( exists $self->{stderr} )
    {
	open STDERR, ">&", delete $self->{stderr}
	  or die( "error restoring STDERR\n" );
    }
}

1;
