package MakeWorld::Mogrify::Patch;

use strict;
use warnings;

use File::Spec::Functions qw[ catfile ];

use MakeWorld::Global;
use MakeWorld::Utils qw[ find_exe ];
use MakeWorld::Check qw[ check %BOOLEAN ];
use MakeWorld::Run;
use MakeWorld::Log;


################################################################################
# execute a command

sub verify {

    my ( $class, $opts, $need_hook ) = @_;

    return check( { patch => { required => 1 },
		    hook => { required => $need_hook },
		    exe  => { required => 0 },
		    optional => { required => 0,
				  default => 0,
				  %BOOLEAN },
		    arg => { default => [] },
		  }, $opts );
}

sub mogrify
{
    my ( $class, $pkg, $opt ) = @_;

    my $guard = log_pfx( '<patch>' );

    # find patch file
    ( my $file ) = grep { -f $_ }
		      map { glob(catfile($_,$opt->{patch})) } @{$Opt->srcdir};

    unless ( $file && -f $file )
    {
	return if $opt->{optional};
	log_fatal("non-existent patch file: $opt->{patch}\n" );
    }

    log_notice( "patch" );

    if ( ! defined $opt->{exe} ) {

	$opt->{exe} = $Opt->patch_cmd;
	$opt->{arg} = [ '-p1' ] unless @{$opt->{arg}} ;
	push @{$opt->{arg}}, '-i';
    }

    my @cmd = ( $opt->{exe}, @{$opt->{arg}}, $file );

    log_info( join(' ', @cmd ) );

    run( @cmd )
      or log_fatal( "error patching\n" );

    return;
}

1;
