package MakeWorld::VarStack;

use strict;
use warnings;

use Storable qw[ dclone ];

use MakeWorld::Log;


my @Stack;

my $id = 0;


sub new {

    my ( $obj ) = @_;

    my $class = ref $obj || $obj;

    my $self = bless( (ref $obj ? dclone($obj) : { var => {} }), $class);

    $self->{_parent} = $obj
	if ref $obj;

    push @Stack, $self;

    return $self;
}

sub DESTROY {

    pop @Stack;

}

sub vars { return $_[0]->{var} }

sub value  { return $_[0]->{var}{$_[1]} }

sub exists { CORE::exists $_[0]->{var}{$_[1]} }

sub set {

    my ( $self, %vars ) = @_;

    my ( $key, $val );
    $self->{var}{$key} = $val while ($key, $val ) = each %vars;

    return $self;
}

sub dump {

    my ( $self, @vars ) = @_;

    @vars = keys %{$self->{var}} unless @vars;

    log_debug( "$_: " . ($self->exists($_) ? $self->value($_) : 'undef') . "\n" )
	foreach @vars;

    return;
}

sub parent {

    die( "internal error: no stack parent\n" )
      if (@_ == 1 || ! $_[-1]) && ! exists $_[0]->{_parent};

    return $_[0]->{_parent};
}

1;
